@@ -11,6 +11,7 @@
 
 #include "ArchHandler.h"
 #include "File.h"
+#include "MachONormalizedFileBinaryUtils.h"
 #include "MachOPasses.h"
 #include "lld/Core/DefinedAtom.h"
 #include "lld/Core/File.h"
@@ -29,7 +30,7 @@ namespace mach_o {
 ///
 class ObjCImageInfoAtom : public SimpleDefinedAtom {
 public:
-  ObjCImageInfoAtom(const File &file,
+  ObjCImageInfoAtom(const File &file, bool isBig,
                     MachOLinkingContext::ObjCConstraint objCConstraint,
                     uint32_t swiftVersion)
       : SimpleDefinedAtom(file) {
@@ -54,6 +55,8 @@ public:
     }
 
     Data.info.flags |= (swiftVersion << 8);
+
+    normalized::write32(Data.bytes + 4, Data.info.flags, isBig);
   }
 
   ~ObjCImageInfoAtom() override = default;
@@ -109,7 +112,8 @@ public:
 private:
 
   const DefinedAtom* getImageInfo() {
-    return new (_file.allocator()) ObjCImageInfoAtom(_file,
+    bool IsBig = MachOLinkingContext::isBigEndian(_ctx.arch());
+    return new (_file.allocator()) ObjCImageInfoAtom(_file, IsBig,
                                                      _ctx.objcConstraint(),
                                                      _ctx.swiftVersion());
   }
